//============ Siemens S45 - Abfrage (Test-Code)===============================
// Code zur Handy-Ansteurung  -> um Abfragen / Antworten auszuwerten!!!
// verison: 03.05.04

#include <avr/io.h>
#include <avr/signal.h>
#include <avr/interrupt.h>
#include <stdio.h>
#include <string.h>

#define CR 0x0D			// ASCII Code for Carriage Return (0x0D = 13)

//Variablen
int i=0;

int dialed=0;
int selnumber=0;

unsigned char cn=0;				// char counter
unsigned char RecBuffer[35];		// Buffer for received chars

//Funktionsprototypen
void MobilePhoneInit(void);
void Dial(void);
void selectNumber(void); 

//============== ACHTUNG ==============================================
// Fuse Bits setzen! -> zur Einstellung des verwendeten Clock-Signals!
// -> Extern oder Intern!
// Extern: CKSEL 3...0	-> 0111 ... fr 3-8MHz
// Intern: CKSEL 3...0  -> 0001 ... fr 1MHz
//						   0011 ... fr 4MHz
// (bei Intern: Bits SUT 1..0 -> 01 ... fr "Fast rising Power", 
//							     10 ... fr "Slowly rising Power"
//=====================================================================

/***********************************************************
	Initialisierung des USARTs des Atmega8
***********************************************************/

#define F_CPU 8000000
#define USART_BAUD_RATE 9600	// 9600 (oder 19200!?)
#define USART_BAUD_SELECT (F_CPU/(USART_BAUD_RATE*16l)-1)


void USART_Init(void) {
    UCSRB = _BV(RXCIE) | _BV(TXCIE) | _BV(RXEN) | _BV(TXEN);	
    UBRRL = (unsigned char) USART_BAUD_SELECT;
}

void USART_transmit (unsigned char c) {
	while (!(UCSRA & (1<<UDRE))) {}
	UDR = c;
}

void USART_transmit_string (unsigned char *string) {
    while (!(UCSRA & (1<<UDRE))) {}
	while ( *string)
		USART_transmit (*string++);
}

unsigned char USART_receive (void) {
    while(!(UCSRA & (1<<RXC))) {}
    return UDR;
}


/*****************************************************************	
					signal handler 
*****************************************************************/

// Prioritt 1 (VecNo. 2)
SIGNAL(SIG_INTERRUPT0) {	// signal handler for external interrupt (PD2 INT0)	
	Dial();
}

// Prioritt 2 (VecNo. 3)
SIGNAL(SIG_INTERRUPT1) {	// signal handler for external interrupt (PD3 INT1)
	USART_transmit_string("extInterruptTest");
}

// Prioritt 3 (VecNo. 10)
SIGNAL(SIG_OVERFLOW0) {		//signal handler for tcnt0 overflow interrupt
	i++;
	USART_transmit(i);
}

// Prioritt 4 (VecNo.12)
SIGNAL(SIG_UART_RECV) {
	cn++;
	RecBuffer[cn]=UDR;
	
	if (dialed == 1) {
		selectNumber();
	}
		
	if (selnumber== 1) {
		if (RecBuffer[cn] == 0x31) {
			GICR=0xC0;	// enables External Interrupt Request of INT1 again (+ INT1)
			TIMSK=0x01;	// enables timer0 overflow bit (T0IE0=1)
			selnumber=0;
		}
	}
}

// Prioritt 5 (VecNo. 14)
SIGNAL(SIG_UART_TRANS) {
	return;
}
	

//-------------------------------------------------------------------------

void InitSFRs(void)
{
	//Port D initialization:
	DDRD=0x0D;	//set PD0, PD2 + PD3 as input, PD1->output
	//PORTD=0x0D;	 // Pull Up Resistors set off
	SFIOR=0x04;		// set PUD to disalbe ALL Pull Ups!!! 
		
	//Timer0 initialization:
	// Clock source: System Clock
	TCCR0=0x04; //  set prescaler to 1:256 (10ms step)
	TCNT0=0x62;	//  Preload Timer0 value for 10ms steps (0x62 = 98)
	//TIMSK=0x01;	// enables timer0 overflow bit (T0IE0=1)
	
	// External Interrupt initialization:
	// INT0: ON
	MCUCR=0x0F;	// Rising Edge of INT0 + INT1 generates an interrupt request
	GICR=0x40;	// enables External Interrupt Request of INT0 
}


int main (void) 
{
	InitSFRs();			// initialize the hardware
	USART_Init();		// initialize USART of ATMEGA8
	outp(0x80,SREG);	// enable (all) interrupts
	MobilePhoneInit();	// initialize the mobilephone
	
	pulsecount = 0;
		
			
	for (;;) {}
			
}


void MobilePhoneInit(void)
{
  USART_transmit_string("atv0");
  USART_transmit(CR);
}

void Dial(void)
{
    GICR=0x40;	// disables External Interrupt Request of INT1
	TIMSK=0x00;	// disables timer0 overflow bit (T0IE0=0)	
	dialed = 1;
	selnumber=0; 
	USART_transmit_string("at+cpbr=99");		// Telefonbucheintrag (Pos. 99) auslesen!
	USART_transmit(CR);							// -> Handy antwortet folgendermaen:
}												// +CPBR: 99, "<telnumber>", 129, "<name"


void selectNumber(void) 
{
	int n=0;
	
	while (1) {
		if (n >= 2) {
			USART_transmit(CR);
			selnumber=1;
			dialed=0;
			break;
		}
		if (RecBuffer[cn] == 0x22 || n == 1) { 			// 0x22 ^= '"'
			if (RecBuffer[cn] == 0x22) {
				n++;
				if (n==1) {
					USART_transmit('a');
					USART_transmit('t');
					USART_transmit('d');
					cn++;
				}
			}
			else if (n==1) {
				USART_transmit(RecBuffer[cn]);
			}
			cn++;
		}
	}
}
			
	


